#!/usr/bin/python
# -*- coding: UTF-8 -*-

############################################################################
##                                                                        ##
## Xchat Purple status                                                    ##
##                                                                        ##
## Copyright (C) 2010  Fabien LOISON (www.flogisoft.com)                  ##
##                                                                        ##
## This program is free software: you can redistribute it and/or modify   ##
## it under the terms of the GNU General Public License as published by   ##
## the Free Software Foundation, either version 3 of the License, or      ##
## (at your option) any later version.                                    ##
##                                                                        ##
## This program is distributed in the hope that it will be useful,        ##
## but WITHOUT ANY WARRANTY; without even the implied warranty of         ##
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          ##
## GNU General Public License for more details.                           ##
##                                                                        ##
## You should have received a copy of the GNU General Public License      ##
## along with this program.  If not, see <http://www.gnu.org/licenses/>.  ##
##                                                                        ##
############################################################################


"""Xchat Purple status - permit you to change Xchat status from Purple
(Pidgin, Finch) or to change Purple status from Xchat.

Description:
    If you change your status in Purple, xchat status will be also changed,
    and if you change your status in Xchat, Purple status will be updated.

Xchat commands:
    /away <message> -- sets you as being away on Xchat and Purple.
    /afk -- an alias of /away.
    /busy <message> -- sets you as being away on Xchat (with message set at
        "Busy" by default) and set your status to unavailable on Purple.
    /back <message> -- sets you as being available on Xchat and Purple.
        NOTE: <message> will be set only on Purple.
"""


__version__ = "0.1"
__author__ = "Fabien Loison (www.flogisoft.com)"
__copyright__ = "Copyright © 2010 Fabien LOISON"

__module_name__ = "Xchat Purple status"
__module_version__ = __version__
__module_description__ = __doc__
__module_author__ = __author__


import dbus
import xchat


#Purple status
STATUS_NONE = 0
STATUS_OFFLINE = 1
STATUS_AVAILABLE = 2
STATUS_UNAVAILABLE = 3
STATUS_INVISIBLE = 4
STATUS_AWAY = 5
STATUS_EXTENDED_AWAY = 6
STATUS_MOBILE = 7
STATUS_TUNE = 8

#Prevent infinite loop when changing status
UPDATING_STATUS = False

#Last purple status
LAST_STATUS = STATUS_NONE


class Purple(object):

    """Purple DBus interface"""

    def __init__(self):
        """The constructor"""
        self._bus = dbus.SessionBus()
        self._purple = None

    def _connect(self):
        """Connect to purple DBus interface

        Return True if connected, False else.
        """
        try:
            obj = self._bus.get_object("im.pidgin.purple.PurpleService", "/im/pidgin/purple/PurpleObject")
            self._purple = dbus.Interface(obj, "im.pidgin.purple.PurpleInterface")
        except dbus.exceptions.DBusException:
            self._purple = None
            return False
        else:
            return True

    def get_status(self):
        """Get current status"""
        if self._connect():
            status = self._purple.PurpleSavedstatusGetCurrent()
            status_type = self._purple.PurpleSavedstatusGetType(status)
            status_message = self._purple.PurpleSavedstatusGetMessage(status)
            return {'status': int(status_type), 'message': str(status_message)}
        else:
            return {'status': STATUS_NONE, 'message': ""}

    def set_status(self, status, message=None):
        """Set status

        If message is None, it will be kept unchanged
        """
        if self._connect():
            if message == None:
                current = self.get_status()
                message = current['message']
            status = self._purple.PurpleSavedstatusNew("", status)
            self._purple.PurpleSavedstatusSetMessage(status, message)
            self._purple.PurpleSavedstatusActivate(status)


def xchat_get_networks():
    chan_list = xchat.get_list("channels")
    server_list = []
    for chan in chan_list:
        server_list.append(chan.network)
    server_list = list(set(server_list))
    return server_list


def xchat_set_status(status, message=""):
    """Sets status for all networks"""
    #Get network list
    server_list = xchat_get_networks()
    #set status
    if server_list:
        for server in server_list:
            context = xchat.find_context(server)
            context.command("%s %s" %(status, message))


def xchat_away_cmd(word, word_eol, datas):
    """/AWAY command"""
    global UPDATING_STATUS
    if not UPDATING_STATUS:
        UPDATING_STATUS = True
        purple = Purple()
        if len(word_eol) > 1:
            message = word_eol[1]
            print("STATUS UPDATED: Away (%s)" %(message))
        else:
            message = ""
            print("STATUS UPDATED: Away")
        purple.set_status(STATUS_AWAY, message)
        xchat_set_status("AWAY", message)
        UPDATING_STATUS = False
        return xchat.EAT_ALL
    else:
        return xchat.EAT_NONE


def xchat_back_cmd(word, word_eol, datas):
    """/BACK command"""
    global UPDATING_STATUS
    if not UPDATING_STATUS:
        UPDATING_STATUS = True
        purple = Purple()
        if len(word_eol) > 1:
            message = word_eol[1]
            print("STATUS UPDATED: Available (%s)" %(message))
        else:
            message = ""
            print("STATUS UPDATED: Available")
        purple.set_status(STATUS_AVAILABLE, message)
        xchat_set_status("BACK")
        UPDATING_STATUS = False
        return xchat.EAT_ALL
    else:
        return xchat.EAT_NONE


def xchat_busy_cmd(word, word_eol, datas):
    """/BUSY command"""
    global UPDATING_STATUS
    if not UPDATING_STATUS:
        UPDATING_STATUS = True
        purple = Purple()
        if len(word_eol) > 1:
            message = word_eol[1]
            print("STATUS UPDATED: Unavailable (%s)" %(message))
        else:
            message = ""
            print("STATUS UPDATED: Unavailable")
        purple.set_status(STATUS_UNAVAILABLE, message)
        if message == "":
            message = "Busy"
        xchat_set_status("AWAY", message)
        UPDATING_STATUS = False
        return xchat.EAT_ALL
    else:
        return xchat.EAT_NONE


def check_status(datas):
    """checks if purple status changed"""
    global LAST_STATUS
    global UPDATING_STATUS
    purple = Purple()
    purple_status = purple.get_status()
    if purple_status['status'] != LAST_STATUS:
        UPDATING_STATUS = True
        LAST_STATUS = purple_status['status']
        if purple_status['status'] == STATUS_AVAILABLE:
            xchat_set_status("BACK")
        elif purple_status['status'] == STATUS_UNAVAILABLE \
            and purple_status['message'] == "":
            xchat_set_status("AWAY", "Busy")
        else:
            xchat_set_status("AWAY", purple_status['message'])
        UPDATING_STATUS = False
    #Timer for checking Purple status
    xchat.hook_timer(2000, check_status)


print("\0034%s %s [%s] loaded.\003" %(__module_name__, __module_version__, __copyright__))

#Xchat commands hook
xchat.hook_command("AWAY", xchat_away_cmd)
xchat.hook_command("AFK", xchat_away_cmd)
xchat.hook_command("BACK", xchat_back_cmd)
xchat.hook_command("BUSY", xchat_busy_cmd)

#Init timer (for checking Purple status)
xchat.hook_timer(60000, check_status)


